@@ -37,7 +37,7 @@ const char mainwid::m_seq_to_char[c_seqs_in_set] =
 
 // Constructor
 
-mainwid::mainwid( perform *a_p  ): DrawingArea() 
+mainwid::mainwid( perform *a_p  ): DrawingArea(), seqmenu( a_p ) 
 {    
     using namespace Menu_Helpers;
 
@@ -70,173 +70,11 @@ mainwid::mainwid( perform *a_p  ): DrawingArea()
     m_button_down = false;
     m_moving = false;
 
-    m_menu = NULL;
-
-     set_double_buffered( false );
+    set_double_buffered( false );
 
 } 
 
 
-void
-mainwid::popup_menu( void )
-{
-
-    using namespace Menu_Helpers;
-    
-    if ( m_menu != NULL )
-        delete m_menu;
-    
-    m_menu = manage( new Menu());
-
-    if ( m_mainperf->is_active( m_current_seq )) {
-        m_menu->items().push_back(MenuElem("Edit", slot(*this,&mainwid::seq_edit)));
-    } else {
-        m_menu->items().push_back(MenuElem("New", slot(*this,&mainwid::seq_edit)));
-    }
-    
-
-        
-    m_menu->items().push_back(SeparatorElem());
-
-    if ( m_mainperf->is_active( m_current_seq )) {
-        m_menu->items().push_back(MenuElem("Cut", slot(*this,&mainwid::seq_cut)));
-        m_menu->items().push_back(MenuElem("Copy", slot(*this,&mainwid::seq_copy)));
-    } else {
-        m_menu->items().push_back(MenuElem("Paste", slot(*this,&mainwid::seq_paste)));
-    }
-    
-    if ( m_mainperf->is_active( m_current_seq )) {
-    
-        m_menu->items().push_back(SeparatorElem());
-        m_menu->items().push_back(MenuElem("Clear Song Data", slot(*this,&mainwid::seq_clear_perf)));
-        m_menu->items().push_back(SeparatorElem());
-
-        Menu *menu_buses = manage( new Menu() );
-         
-        m_menu->items().push_back( MenuElem( "Midi Bus", *menu_buses) );
-        
-        /* midi buses */
-        mastermidibus *masterbus = m_mainperf->get_master_midi_bus();
-        for ( int i=0; i< masterbus->get_num_out_buses(); i++ ){
-
-            Menu *menu_channels = manage( new Menu() );
-            
-            menu_buses->items().push_back(MenuElem( masterbus->get_midi_bus_name(i),
-                                                    *menu_channels ));
-            char b[4];
-            
-            /* midi channel menu */
-            for( int j=0; j<16; j++ ){
-                
-                sprintf( b, "%d", j+1 );
-                menu_channels->items().push_back(MenuElem(b, 
-                                                          bind(slot(*this,&mainwid::set_bus_and_midi_channel), 
-                                                               i, j )));
-            }
-        }        
-    }
-    
-    m_menu->popup(0,0);
-
-}
-
-void
-mainwid::set_bus_and_midi_channel( int a_bus, int a_ch )
-{
-    if ( m_mainperf->is_active( m_current_seq )) {
-        m_mainperf->get_sequence( m_current_seq )->set_midi_bus( a_bus );
-        m_mainperf->get_sequence( m_current_seq )->set_midi_channel( a_ch );
-
-        draw_sequence_on_pixmap( m_current_seq  );
-        draw_sequence_pixmap_on_window( m_current_seq );
-    }
-}
-
-
-
-// Menu callback, Lanches Editor Window
-void 
-mainwid::seq_edit(){
-
-    seqedit *seq_edit;
-    
-    if ( m_mainperf->is_active( m_current_seq )) {
-        if ( !m_mainperf->get_sequence( m_current_seq )->get_editing()){
-            seq_edit = new seqedit( m_mainperf->get_sequence( m_current_seq ), 
-                                    m_mainperf, 
-                                    this,
-                                    m_current_seq
-                                    );
-        }
-    }
-    else {
-        this->seq_new();
-        seq_edit = new seqedit( m_mainperf->get_sequence( m_current_seq ), 
-                                m_mainperf, 
-                                this, 
-                                m_current_seq
-                                );
-    }    
-}
-
-// Makes a New sequence 
-void 
-mainwid::seq_new(){
-
-    if ( ! m_mainperf->is_active( m_current_seq )){
-
-	m_mainperf->new_sequence( m_current_seq );
-	draw_sequence_on_pixmap( m_current_seq  );
-	draw_sequence_pixmap_on_window( m_current_seq );
-    }
-}
-
-// Copies selected to clipboard sequence */
-void 
-mainwid::seq_copy(){
-
-    if ( m_mainperf->is_active( m_current_seq ))
-	m_clipboard = *(m_mainperf->get_sequence( m_current_seq ));
-}
-
-// Deletes and Copies to Clipboard */
-void 
-mainwid::seq_cut(){
-
-    if ( m_mainperf->is_active( m_current_seq ) &&
-		 !m_mainperf->is_sequence_in_edit( m_current_seq ) ){
-
-	m_clipboard = *(m_mainperf->get_sequence( m_current_seq ));
-	m_mainperf->delete_sequence( m_current_seq );
-	draw_sequence_on_pixmap( m_current_seq  );
-	draw_sequence_pixmap_on_window( m_current_seq );
-    }
-}
-
-// Puts clipboard into location
-void 
-mainwid::seq_paste(){
-    
-    if ( ! m_mainperf->is_active( m_current_seq )){
-	
-	m_mainperf->new_sequence( m_current_seq  );
-	*(m_mainperf->get_sequence( m_current_seq )) = m_clipboard;
-	
-	draw_sequence_on_pixmap( m_current_seq  );
-	draw_sequence_pixmap_on_window( m_current_seq );
-    }
-}
-
-
-void 
-mainwid::seq_clear_perf(){
-
-	if ( m_mainperf->is_active( m_current_seq )){
-	
-		m_mainperf->clear_sequence_triggers( m_current_seq  );
-	}
-}
-
 // GTK, on realize of window, init shiz
 void 
 mainwid::on_realize()
@@ -373,7 +211,7 @@ mainwid::draw_sequence_on_pixmap( int a_seq )
 	  
 	    char str[20];
 	    sprintf( str, 
-		     "%1d-%02d    %2d/%2d", 
+		     "%d-%d %d/%d", 
 		     seq->get_midi_bus(), 
 		     seq->get_midi_channel()+1,
 		     seq->get_bpm(), seq->get_bw() );
@@ -499,7 +337,13 @@ mainwid::draw_sequence_pixmap_on_window( int a_seq )
     
 }
 
-  
+void
+mainwid::redraw( int a_sequence )
+{
+    draw_sequence_on_pixmap( a_sequence );
+    draw_sequence_pixmap_on_window( a_sequence );
+}
+
 void 
 mainwid::update_markers( int a_ticks )
 {
@@ -512,14 +356,15 @@ void
 mainwid::draw_marker_on_sequence( int a_seq, int a_tick )
 {
 
+   	if ( m_mainperf->is_dirty_main(a_seq ) ){
+		update_sequence_on_window( a_seq );
+	}
+
+ 
     if ( m_mainperf->is_active( a_seq )){
 
 	sequence *seq = m_mainperf->get_sequence( a_seq );
 
-	if ( seq->is_dirty_main() ){
-		update_sequence_on_window( a_seq );
-	}
-
 	int i =  (a_seq / c_mainwnd_rows) % c_mainwnd_cols;
 	int j =  a_seq % c_mainwnd_rows;
 
@@ -530,8 +375,9 @@ mainwid::draw_marker_on_sequence( int a_seq, int a_tick )
 
 	int rectangle_x = base_x + c_text_x - 1;
 	int rectangle_y = base_y + c_text_y + c_text_x - 1;
-	
+        
 	int length = seq->get_length( );
+        a_tick += (length - seq->get_trigger_offset( ));
 	a_tick %= length;
 
 	long tick_x = a_tick * c_seqarea_seq_x / length;
