@@ -29,7 +29,6 @@ midibus::midibus( int a_localclient,
 		  const char *a_port_name,
 		  char a_id, int a_queue )
 {
- 
     /* set members */
     m_local_addr_client = a_localclient;
     m_dest_addr_client = a_destclient;
@@ -38,7 +37,8 @@ midibus::midibus( int a_localclient,
     m_queue          = a_queue;
 
     m_id = a_id;
-    m_clocking = false;
+    m_clock_type = e_clock_off;
+    m_inputing = false;
 
     /* copy names */
     char tmp[60];
@@ -49,10 +49,32 @@ midibus::midibus( int a_localclient,
 	      a_port_name );
 
     m_name = tmp;
+}
 
+midibus::midibus( int a_localclient,
+		  snd_seq_t *a_seq, 
+		  char a_id, int a_queue )
+{
+    /* set members */
+    m_local_addr_client = a_localclient;
+    m_seq            = a_seq;
+    m_queue          = a_queue;
+
+    m_id = a_id;
+    m_clock_type = e_clock_off;
+    m_inputing = false;
+
+    /* copy names */
+    char tmp[60];
+    snprintf( tmp, 59, "[%d] seq24 %d",
+	      m_id,
+	      m_id );
+
+    m_name = tmp;
 }
 
 
+int midibus::m_clock_mod = 16 * 4;
 
 void 
 midibus::lock( )
@@ -78,7 +100,7 @@ bool midibus::init_out( )
 
     /* create ports */
     ret = snd_seq_create_simple_port(m_seq, 
-				     "seq24 out",
+				     m_name.c_str(),
 			     	     SND_SEQ_PORT_CAP_NO_EXPORT |
 				     SND_SEQ_PORT_CAP_READ,
 				     SND_SEQ_PORT_TYPE_MIDI_GENERIC |
@@ -104,6 +126,29 @@ bool midibus::init_out( )
 }
 
 
+bool midibus::init_out_sub( )
+{
+    /* temp return */
+    int ret;
+
+    /* create ports */
+    ret = snd_seq_create_simple_port(m_seq, 
+				     m_name.c_str(),
+				     SND_SEQ_PORT_CAP_READ |
+                     SND_SEQ_PORT_CAP_SUBS_READ,
+				     SND_SEQ_PORT_TYPE_MIDI_GENERIC |
+				     SND_SEQ_PORT_TYPE_APPLICATION );
+    m_local_addr_port = ret;
+
+    if ( ret < 0 ){
+        printf( "snd_seq_create_simple_port(write) error\n");
+        return false;
+    }
+
+    return true;
+}
+
+
 
 bool midibus::init_in( )
 {
@@ -155,6 +200,66 @@ bool midibus::init_in( )
 }
 
 
+bool midibus::init_in_sub( )
+{
+    /* temp return */
+    int ret;
+
+    /* create ports */
+    ret = snd_seq_create_simple_port(m_seq, 
+				     "seq24 in",
+				     SND_SEQ_PORT_CAP_WRITE |
+                     SND_SEQ_PORT_CAP_SUBS_WRITE,
+				     SND_SEQ_PORT_TYPE_MIDI_GENERIC |
+				     SND_SEQ_PORT_TYPE_APPLICATION );
+    m_local_addr_port = ret;
+
+    if ( ret < 0 ){
+        printf( "snd_seq_create_simple_port(write) error\n");
+        return false;
+    }
+
+    return true;
+}
+
+
+
+
+bool midibus::deinit_in( )
+{
+
+    /* temp return */
+    int ret;
+    
+    snd_seq_port_subscribe_t *subs;
+    snd_seq_port_subscribe_alloca(&subs);
+    snd_seq_addr_t sender, dest;
+
+    /* the destinatino port is actually our local port */
+    sender.client = m_dest_addr_client; 
+    sender.port = m_dest_addr_port;   
+    dest.client = m_local_addr_client;
+    dest.port = m_local_addr_port;
+
+    /* set in and out ports */
+    snd_seq_port_subscribe_set_sender(subs, &sender);
+    snd_seq_port_subscribe_set_dest(subs, &dest);
+
+    /* use the master queue, and get ticks */
+    snd_seq_port_subscribe_set_queue(subs, m_queue);
+    snd_seq_port_subscribe_set_time_update(subs, 1);
+
+    /* subscribe */
+    ret = snd_seq_unsubscribe_port(m_seq, subs);
+
+    if ( ret < 0 ){
+        printf( "snd_seq_unsubscribe_port(%d:%d) error\n", m_dest_addr_client, m_dest_addr_port);
+        return false;
+    }
+
+    return true;
+}
+
 
 int
 midibus::get_id( )
@@ -300,7 +405,87 @@ midibus::flush()
 } 
 
 
+void 
+midibus::init_clock( long a_tick )
+{
+    if ( m_clock_type == e_clock_pos && a_tick != 0)
+    {
+        continue_from( a_tick );
+    }
+    else 
+    if ( m_clock_type == e_clock_mod || a_tick == 0)
+    {
+        start();
 
+        long clock_mod_ticks = (c_ppqn / 4) * m_clock_mod;
+        long leftover = ( a_tick % clock_mod_ticks );
+        long starting_tick = a_tick - leftover;
+
+        /* was there anything left?, then wait for next beat (16th note) to start clocking */
+        if ( leftover > 0)
+        {
+            starting_tick += clock_mod_ticks;
+        }
+        //printf ( "continue_from leftover[%ld] starting_tick[%ld]\n", leftover, starting_tick );
+
+        m_lasttick = starting_tick - 1;    
+
+
+    }
+}
+
+void
+midibus::continue_from( long a_tick )
+{
+    /* tell the device that we are going to start at a certain position */
+    long pp16th = (c_ppqn / 4);
+
+    long leftover = ( a_tick % pp16th );
+    long beats = ( a_tick / pp16th );
+
+    long starting_tick = a_tick - leftover;
+
+    /* was there anything left?, then wait for next beat (16th note) to start clocking */
+    if ( leftover > 0)
+    {
+        starting_tick += pp16th;
+    }
+    //printf ( "continue_from leftover[%ld] starting_tick[%ld]\n", leftover, starting_tick );
+
+    m_lasttick = starting_tick - 1;    
+
+    if ( m_clock_type != e_clock_off )
+    {
+        //printf( "control value %ld\n",  beats);
+
+        snd_seq_event_t evc;
+        snd_seq_event_t ev;
+
+        ev.type = SND_SEQ_EVENT_CONTINUE;
+        evc.type = SND_SEQ_EVENT_SONGPOS;
+        evc.data.control.value = beats;
+        snd_seq_ev_set_fixed( &ev );    
+        snd_seq_ev_set_fixed( &evc );
+
+        snd_seq_ev_set_priority( &ev, 1 );
+        snd_seq_ev_set_priority( &evc, 1 );
+
+        /* set source */
+        snd_seq_ev_set_source(&evc, m_local_addr_port );
+        snd_seq_ev_set_subs(&evc);
+        snd_seq_ev_set_source(&ev, m_local_addr_port );
+        snd_seq_ev_set_subs(&ev);
+
+        // its immediate 
+        snd_seq_ev_set_direct( &ev );
+        snd_seq_ev_set_direct( &evc );
+
+        /* pump it into the queue */
+        snd_seq_event_output(m_seq, &evc);
+        flush();
+        snd_seq_event_output(m_seq, &ev);
+    }
+}
 
 
 /* gets it a runnin */
@@ -309,7 +494,7 @@ midibus::start()
 {
     m_lasttick = -1;
     
-    if ( m_clocking ){ 
+    if ( m_clock_type != e_clock_off ){ 
 	
 	snd_seq_event_t ev;
 	
@@ -329,31 +514,58 @@ midibus::start()
 	snd_seq_event_output(m_seq, &ev);
 
     }
-
 }
 
 
 void 
-midibus::set_clock( bool a_clocking )
+midibus::set_clock( clock_e a_clock_type )
 {
-    m_clocking = a_clocking;
+    m_clock_type = a_clock_type;
+}
+
+
+clock_e
+midibus::get_clock( )
+{
+    return m_clock_type;
+}
+
+
+
+
+void 
+midibus::set_input( bool a_inputing )
+{
+    if ( m_inputing != a_inputing ){
+
+        m_inputing = a_inputing;
+
+        if (m_inputing){
+            init_in();
+        }
+        else
+        {
+            deinit_in();
+        }
+    }    
 }
 
 
 bool 
-midibus::get_clock( )
+midibus::get_input( )
 {
-    return m_clocking;
+    return m_inputing;
 }
 
 
+
 void 
 midibus::stop()
 {
 
     m_lasttick = -1;
 
-    if ( m_clocking ){   
+    if ( m_clock_type != e_clock_off ){   
 	
 	snd_seq_event_t ev;
 	
@@ -373,10 +585,11 @@ midibus::stop()
 	snd_seq_event_output(m_seq, &ev);
 	
     }
-
 }
 
 
+
+
 // generates midi clock
 void
 midibus::clock( long a_tick )
@@ -384,13 +597,13 @@ midibus::clock( long a_tick )
 
     lock();
 
-    if ( m_clocking ){
+    if ( m_clock_type != e_clock_off ){
 
 	bool done = false;
 	
 	long uptotick = a_tick;
 	
-	if ( m_lasttick == uptotick )
+	if ( m_lasttick >= uptotick )
 	    done = true;
 	
 	while ( !done ){
@@ -424,6 +637,7 @@ midibus::clock( long a_tick )
 		
 		/* pump it into the queue */
 		snd_seq_event_output(m_seq, &ev);
+
 	    }
 	}
 	/* and send out */
@@ -490,21 +704,48 @@ mastermidibus::start()
      unlock();
 }
 
+
+/* gets it a runnin */
+    void
+mastermidibus::continue_from( long a_tick)
+{
+    lock();
+
+    /* start timer */
+    snd_seq_start_queue( m_alsa_seq, m_queue, NULL );
+
+    for ( int i=0; i < m_num_out_buses; i++ )
+        m_buses_out[i]->continue_from( a_tick );
+
+    unlock();
+}
+
+void
+mastermidibus::init_clock( long a_tick )
+{
+    lock();
+
+    for ( int i=0; i < m_num_out_buses; i++ )
+        m_buses_out[i]->init_clock( a_tick );
+
+    unlock();
+}    
+
 void 
 mastermidibus::stop()
 {
     lock();
-         
+
+    for ( int i=0; i < m_num_out_buses; i++ )
+        m_buses_out[i]->stop();
+
+
     snd_seq_drain_output( m_alsa_seq );
     snd_seq_sync_output_queue( m_alsa_seq );
 
     /* start timer */
     snd_seq_stop_queue( m_alsa_seq, m_queue, NULL );
-
-    for ( int i=0; i < m_num_out_buses; i++ )
-	m_buses_out[i]->stop();
-
-     unlock();
+    unlock();
 }
 
 
@@ -584,111 +825,152 @@ mastermidibus::mastermidibus()
 {
     /* temp return */
     int ret;
-
+    
     /* set initial number buses */
     m_num_out_buses = 0;
     m_num_in_buses = 0;
 
-	for( int i=0; i<c_maxBuses; ++i ){
-		m_buses_in_active[i] = false;	
-		m_buses_out_active[i] = false;
-		m_buses_in_init[i] = false;	
-		m_buses_out_init[i] = false;
-	}
-
+    for( int i=0; i<c_maxBuses; ++i ){
+        m_buses_in_active[i] = false;	
+        m_buses_out_active[i] = false;
+        m_buses_in_init[i] = false;	
+        m_buses_out_init[i] = false;
+    }
+    
     /* open the sequencer client */
     ret = snd_seq_open(&m_alsa_seq, "default",  SND_SEQ_OPEN_DUPLEX, 0);
-
+    
     if ( ret < 0 ){
 	printf( "snd_seq_open() error\n");
 	exit(1);
     }
-  
+
     /* set our clients name */
     snd_seq_set_client_name(m_alsa_seq, "seq24");
-  
+
     /* set up our clients queue */
     m_queue = snd_seq_alloc_queue( m_alsa_seq );
 
+}
+
+void
+mastermidibus::init( )
+{
+    
     /* client info */
     snd_seq_client_info_t *cinfo;
     /* port info */
     snd_seq_port_info_t *pinfo;
-	
+
     int client;
-  
+
     snd_seq_client_info_alloca(&cinfo);
     snd_seq_client_info_set_client(cinfo, -1);
 
-
-    /* while the next client one the sequencer is avaiable */
-    while (snd_seq_query_next_client(m_alsa_seq, cinfo) >= 0){
-
-	/* get client from cinfo */
-	client = snd_seq_client_info_get_client(cinfo);
+    //printf( "global_ports %d\n", global_manual_alsa_ports );
     
-	/* fill pinfo */
-	snd_seq_port_info_alloca(&pinfo);
-	snd_seq_port_info_set_client(pinfo, client);
-	snd_seq_port_info_set_port(pinfo, -1);
+    if ( global_manual_alsa_ports )
+    {
 
-	/* while the next port is avail */
-	while (snd_seq_query_next_port(m_alsa_seq, pinfo) >= 0 ){
-			
-	    /* get its capability */
-	    int cap =  snd_seq_port_info_get_capability(pinfo);
-	    
-	    if ( snd_seq_client_id( m_alsa_seq ) != snd_seq_port_info_get_client(pinfo) &&
-		 snd_seq_port_info_get_client(pinfo) != SND_SEQ_CLIENT_SYSTEM){
+        int num_buses = 16;
+        
+        for( int i=0; i<num_buses; ++i )
+        {
 
-		/* the outs */
-		if ( (cap & SND_SEQ_PORT_CAP_SUBS_WRITE) != 0 &&
-		     snd_seq_client_id( m_alsa_seq ) != snd_seq_port_info_get_client(pinfo)){
-		    
-		    m_buses_out[m_num_out_buses] = 
-			new midibus( snd_seq_client_id( m_alsa_seq ),
-				     snd_seq_port_info_get_client(pinfo),
-				     snd_seq_port_info_get_port(pinfo),
-				     m_alsa_seq,
-				     snd_seq_client_info_get_name(cinfo),
-				     snd_seq_port_info_get_name(pinfo),
-				     m_num_out_buses, m_queue );
-		    
-		    if ( m_buses_out[m_num_out_buses]->init_out() ){
-                m_buses_out_active[m_num_out_buses] = true;		
-                m_buses_out_init[m_num_out_buses] = true;
-            } else {
-                m_buses_out_init[m_num_out_buses] = true;	
+            m_buses_out[i] = 
+                new midibus( snd_seq_client_id( m_alsa_seq ), m_alsa_seq, i+1, m_queue );
+
+            m_buses_out[i]->init_out_sub();
+            m_buses_out_active[i] = true;		
+            m_buses_out_init[i] = true;
+        }
+
+        m_num_out_buses = num_buses;
+
+        /* only one in */
+        m_buses_in[0] = 
+            new midibus( snd_seq_client_id( m_alsa_seq ),
+                    m_alsa_seq,
+                    m_num_in_buses, m_queue);
+
+        m_buses_in[0]->init_in_sub();
+        m_buses_in_active[0] = true;	
+        m_buses_in_init[0] = true;
+        m_num_in_buses = 1;
+
+
+    }
+    else 
+    {
+        /* while the next client one the sequencer is avaiable */
+        while (snd_seq_query_next_client(m_alsa_seq, cinfo) >= 0){
+
+            /* get client from cinfo */
+            client = snd_seq_client_info_get_client(cinfo);
+
+            /* fill pinfo */
+            snd_seq_port_info_alloca(&pinfo);
+            snd_seq_port_info_set_client(pinfo, client);
+            snd_seq_port_info_set_port(pinfo, -1);
+
+            /* while the next port is avail */
+            while (snd_seq_query_next_port(m_alsa_seq, pinfo) >= 0 ){
+
+                /* get its capability */
+                int cap =  snd_seq_port_info_get_capability(pinfo);
+
+                if ( snd_seq_client_id( m_alsa_seq ) != snd_seq_port_info_get_client(pinfo) &&
+                        snd_seq_port_info_get_client(pinfo) != SND_SEQ_CLIENT_SYSTEM){
+
+                    /* the outs */
+                    if ( (cap & SND_SEQ_PORT_CAP_SUBS_WRITE) != 0 &&
+                            snd_seq_client_id( m_alsa_seq ) != snd_seq_port_info_get_client(pinfo)){
+
+                        m_buses_out[m_num_out_buses] = 
+                            new midibus( snd_seq_client_id( m_alsa_seq ),
+                                    snd_seq_port_info_get_client(pinfo),
+                                    snd_seq_port_info_get_port(pinfo),
+                                    m_alsa_seq,
+                                    snd_seq_client_info_get_name(cinfo),
+                                    snd_seq_port_info_get_name(pinfo),
+                                    m_num_out_buses, m_queue );
+
+                        if ( m_buses_out[m_num_out_buses]->init_out() ){
+                            m_buses_out_active[m_num_out_buses] = true;		
+                            m_buses_out_init[m_num_out_buses] = true;
+                        } else {
+                            m_buses_out_init[m_num_out_buses] = true;	
+                        }
+
+                        m_num_out_buses++;
+                    }	
+
+                    /* the ins */
+                    if ( (cap & SND_SEQ_PORT_CAP_SUBS_READ) != 0 &&
+                            snd_seq_client_id( m_alsa_seq ) != snd_seq_port_info_get_client(pinfo)){
+
+                        m_buses_in[m_num_in_buses] = 
+                            new midibus( snd_seq_client_id( m_alsa_seq ),
+                                    snd_seq_port_info_get_client(pinfo),
+                                    snd_seq_port_info_get_port(pinfo),
+                                    m_alsa_seq,
+                                    snd_seq_client_info_get_name(cinfo),
+                                    snd_seq_port_info_get_name(pinfo),
+                                    m_num_in_buses, m_queue);
+
+                        //if ( m_buses_in[m_num_in_buses]->init_in() ){
+                        m_buses_in_active[m_num_in_buses] = true;	
+                        m_buses_in_init[m_num_in_buses] = true;
+                        //} else {
+                        //m_buses_in_init[m_num_in_buses] = true;	
+                        //}
+                        m_num_in_buses++;
+                    }	
+                }
             }
 
-		    m_num_out_buses++;
-		}	
-		
-		/* the ins */
-		if ( (cap & SND_SEQ_PORT_CAP_SUBS_READ) != 0 &&
-		     snd_seq_client_id( m_alsa_seq ) != snd_seq_port_info_get_client(pinfo)){
-		    
-		    m_buses_in[m_num_in_buses] = 
-			new midibus( snd_seq_client_id( m_alsa_seq ),
-				     snd_seq_port_info_get_client(pinfo),
-				     snd_seq_port_info_get_port(pinfo),
-				     m_alsa_seq,
-				     snd_seq_client_info_get_name(cinfo),
-				     snd_seq_port_info_get_name(pinfo),
-				      m_num_in_buses, m_queue);
-		    	
-			if ( m_buses_in[m_num_in_buses]->init_in() ){
-                m_buses_in_active[m_num_in_buses] = true;	
-                m_buses_in_init[m_num_in_buses] = true;
-            } else {
-                m_buses_in_init[m_num_in_buses] = true;	
-            }
-		    m_num_in_buses++;
-		}	
-	    }
-	}
-
-    } /* end loop for clients */
+        } /* end loop for clients */
+    }
 
     set_bpm( c_bpm );
     set_ppqn( c_ppqn );
@@ -704,28 +986,26 @@ mastermidibus::mastermidibus()
 
     /* get descriptors */
     snd_seq_poll_descriptors(m_alsa_seq,  
-			     m_poll_descriptors, 
-			     m_num_poll_descriptors, 
-			     POLLIN);
-    
+            m_poll_descriptors, 
+            m_num_poll_descriptors, 
+            POLLIN);
+
     set_sequence_input( false, NULL );
 
     /* sizes */
     snd_seq_set_output_buffer_size(m_alsa_seq, c_midibus_output_size ); 
     snd_seq_set_input_buffer_size(m_alsa_seq, c_midibus_input_size ); 
 
-		 
-	m_bus_announce = 
-			new midibus( snd_seq_client_id( m_alsa_seq ),
-						 SND_SEQ_CLIENT_SYSTEM,
-						 SND_SEQ_PORT_SYSTEM_ANNOUNCE,
-						 m_alsa_seq,
-						 "system","annouce",
-						 0, m_queue);
-		    
-	m_bus_announce->init_in();
 
+    m_bus_announce = 
+        new midibus( snd_seq_client_id( m_alsa_seq ),
+                SND_SEQ_CLIENT_SYSTEM,
+                SND_SEQ_PORT_SYSTEM_ANNOUNCE,
+                m_alsa_seq,
+                "system","annouce",
+                0, m_queue);
 
+    m_bus_announce->set_input(true);   
 }
       
 mastermidibus::~mastermidibus()
@@ -774,27 +1054,59 @@ mastermidibus::play( unsigned char a_bus, event *a_e24, unsigned char a_channel
 
 
 void 
-mastermidibus::set_clock( unsigned char a_bus, bool a_clocking )
+mastermidibus::set_clock( unsigned char a_bus, clock_e a_clock_type )
 {
     lock();
     if ( m_buses_out_active[a_bus] && a_bus < m_num_out_buses ){
-        m_buses_out[a_bus]->set_clock( a_clocking );
+        m_buses_out[a_bus]->set_clock( a_clock_type );
+    }
+    unlock();
+}
+
+clock_e 
+mastermidibus::get_clock( unsigned char a_bus )
+{
+	if ( m_buses_out_active[a_bus] && a_bus < m_num_out_buses ){
+		return m_buses_out[a_bus]->get_clock();
+	}
+	return e_clock_off;
+}
+
+void 
+midibus::set_clock_mod( int a_clock_mod )
+{
+    m_clock_mod = a_clock_mod;
+}
+
+int 
+midibus::get_clock_mod( void )
+{
+    return m_clock_mod;
+}
+      
+
+void 
+mastermidibus::set_input( unsigned char a_bus, bool a_inputing )
+{
+    lock();
+    if ( m_buses_in_active[a_bus] && a_bus < m_num_in_buses ){
+        m_buses_in[a_bus]->set_input( a_inputing );
     }
     unlock();
 }
 
 bool 
-mastermidibus::get_clock( unsigned char a_bus )
+mastermidibus::get_input( unsigned char a_bus )
 {
-	if ( m_buses_out_active[a_bus] && a_bus < m_num_out_buses ){
-		return m_buses_out[a_bus]->get_clock();
+	if ( m_buses_in_active[a_bus] && a_bus < m_num_in_buses ){
+		return m_buses_in[a_bus]->get_input();
 	}
 	return false;
 }
 
 
 string
-mastermidibus::get_midi_bus_name( int a_bus )
+mastermidibus::get_midi_out_bus_name( int a_bus )
 {
 	if ( m_buses_out_active[a_bus] && a_bus < m_num_out_buses ){
 		return m_buses_out[a_bus]->get_name();
@@ -819,6 +1131,31 @@ mastermidibus::get_midi_bus_name( int a_bus )
 }
 
 
+string
+mastermidibus::get_midi_in_bus_name( int a_bus )
+{
+	if ( m_buses_in_active[a_bus] && a_bus < m_num_in_buses ){
+		return m_buses_in[a_bus]->get_name();
+	}
+	
+	/* copy names */
+	char tmp[60];
+
+	if ( m_buses_in_init[a_bus] ){
+		snprintf( tmp, 59, "[%d] %d:%d (disconnected)",
+				  a_bus,
+				  m_buses_in[a_bus]->get_client(),
+				  m_buses_in[a_bus]->get_port() );
+	} else {
+		snprintf( tmp, 59, "[%d] (unconnected)",
+				  a_bus );
+	}
+
+	string ret = tmp;
+	return ret;	
+	
+}
+
 
 void 
 mastermidibus::print()
@@ -837,6 +1174,12 @@ mastermidibus::get_num_out_buses()
 }
 
 
+int 
+mastermidibus::get_num_in_buses()
+{
+    return m_num_in_buses;
+}
+
 int
 mastermidibus::poll_for_midi( )
 {
@@ -865,87 +1208,111 @@ mastermidibus::is_more_input( ){
 void 
 mastermidibus::port_start( int a_client, int a_port )
 {
-	lock();
-
-	bool replacement = false;
-	int bus_slot = m_num_out_buses;
-
-	for( int i=0; i< m_num_out_buses; i++ ){
-
-		if( m_buses_out[i]->get_client() == a_client  &&
-			m_buses_out[i]->get_port() == a_port &&
-			m_buses_out_active[i] == false ){
-
-			replacement = true;
-			bus_slot = i;
-		}
-	}
-
-	/* client info */
+    lock();
+    
+ 
+    
+    /* client info */
     snd_seq_client_info_t *cinfo;
     snd_seq_client_info_alloca(&cinfo);
-	snd_seq_get_any_client_info( m_alsa_seq, a_client, cinfo );  
-
+    snd_seq_get_any_client_info( m_alsa_seq, a_client, cinfo );  
+    
     /* port info */
     snd_seq_port_info_t *pinfo;
-	 
-	/* fill pinfo */
-	snd_seq_port_info_alloca(&pinfo);
-	snd_seq_get_any_port_info( m_alsa_seq, a_client, a_port, pinfo );  
+    
+    /* fill pinfo */
+    snd_seq_port_info_alloca(&pinfo);
+    snd_seq_get_any_port_info( m_alsa_seq, a_client, a_port, pinfo );  
+    
+    
+    /* get its capability */
+    int cap =  snd_seq_port_info_get_capability(pinfo);
+    
+    if ( snd_seq_client_id( m_alsa_seq ) != snd_seq_port_info_get_client(pinfo)){
+        
+        /* the outs */
+        if ( (cap & (SND_SEQ_PORT_CAP_SUBS_WRITE | SND_SEQ_PORT_CAP_WRITE ))
+             == (SND_SEQ_PORT_CAP_SUBS_WRITE | SND_SEQ_PORT_CAP_WRITE )
+             && snd_seq_client_id( m_alsa_seq ) != snd_seq_port_info_get_client(pinfo)){
 
-			
-	/* get its capability */
-	int cap =  snd_seq_port_info_get_capability(pinfo);
-	    
-	if ( snd_seq_client_id( m_alsa_seq ) != snd_seq_port_info_get_client(pinfo)){
 
-		/* the outs */
-		if ( (cap & (SND_SEQ_PORT_CAP_SUBS_WRITE | SND_SEQ_PORT_CAP_WRITE ))
-			 == (SND_SEQ_PORT_CAP_SUBS_WRITE | SND_SEQ_PORT_CAP_WRITE )
-			 && snd_seq_client_id( m_alsa_seq ) != snd_seq_port_info_get_client(pinfo)){
-		    
-		    m_buses_out[bus_slot] = 
-				new midibus( snd_seq_client_id( m_alsa_seq ),
-							 snd_seq_port_info_get_client(pinfo),
-							 snd_seq_port_info_get_port(pinfo),
-							 m_alsa_seq,
-							 snd_seq_client_info_get_name(cinfo),
-							 snd_seq_port_info_get_name(pinfo),
-							 m_num_out_buses, m_queue );
-		    
-		    m_buses_out[bus_slot]->init_out();
-			m_buses_out_active[bus_slot] = true;
-			m_buses_out_init[bus_slot] = true;
-			
-			if ( !replacement ){
-				m_num_out_buses++;
-			}
-		}	
-		
-		/* the ins */
-		if ( (cap & (SND_SEQ_PORT_CAP_SUBS_READ | SND_SEQ_PORT_CAP_READ ))
-			 == (SND_SEQ_PORT_CAP_SUBS_READ | SND_SEQ_PORT_CAP_READ )
-			 && snd_seq_client_id( m_alsa_seq ) != snd_seq_port_info_get_client(pinfo)){
-	    
-		    m_buses_in[m_num_in_buses] = 
-			new midibus( snd_seq_client_id( m_alsa_seq ),
-				     snd_seq_port_info_get_client(pinfo),
-				     snd_seq_port_info_get_port(pinfo),
-				     m_alsa_seq,
-				     snd_seq_client_info_get_name(cinfo),
-				     snd_seq_port_info_get_name(pinfo),
-				      m_num_in_buses, m_queue);
-		    
-		    m_buses_in[m_num_in_buses]->init_in();
-		    m_buses_in_active[m_num_in_buses] = true;
-		    m_buses_in_init[m_num_in_buses] = true;
+            bool replacement = false;
+            int bus_slot = m_num_out_buses;
+            
+            for( int i=0; i< m_num_out_buses; i++ ){
+                
+                if( m_buses_out[i]->get_client() == a_client  &&
+                    m_buses_out[i]->get_port() == a_port &&
+                    m_buses_out_active[i] == false ){
+                    
+                    replacement = true;
+                    bus_slot = i;
+                }
+            }
+            
+            m_buses_out[bus_slot] = 
+                new midibus( snd_seq_client_id( m_alsa_seq ),
+                             snd_seq_port_info_get_client(pinfo),
+                             snd_seq_port_info_get_port(pinfo),
+                             m_alsa_seq,
+                             snd_seq_client_info_get_name(cinfo),
+                             snd_seq_port_info_get_name(pinfo),
+                             m_num_out_buses, m_queue );
+            
+            m_buses_out[bus_slot]->init_out();
+            m_buses_out_active[bus_slot] = true;
+            m_buses_out_init[bus_slot] = true;
+            
+            if ( !replacement ){
+                m_num_out_buses++;
+            }
+        }	
+        
+        /* the ins */
+        if ( (cap & (SND_SEQ_PORT_CAP_SUBS_READ | SND_SEQ_PORT_CAP_READ ))
+             == (SND_SEQ_PORT_CAP_SUBS_READ | SND_SEQ_PORT_CAP_READ )
+             && snd_seq_client_id( m_alsa_seq ) != snd_seq_port_info_get_client(pinfo)){
+            
+            bool replacement = false;
+            int bus_slot = m_num_in_buses;
 
-		    m_num_in_buses++;
-		}	
-	}
-	/* end loop for clients */
+          
+            
+            for( int i=0; i< m_num_in_buses; i++ ){
+                
+                if( m_buses_in[i]->get_client() == a_client  &&
+                    m_buses_in[i]->get_port() == a_port &&
+                    m_buses_in_active[i] == false ){
+                    
+                    replacement = true;
+                    bus_slot = i;
+                }
+            }
 
- 
+            //printf( "in [%d] [%d]\n", replacement, bus_slot );
+            
+            m_buses_in[bus_slot] = 
+                new midibus( snd_seq_client_id( m_alsa_seq ),
+                             snd_seq_port_info_get_client(pinfo),
+                             snd_seq_port_info_get_port(pinfo),
+                             m_alsa_seq,
+                             snd_seq_client_info_get_name(cinfo),
+                             snd_seq_port_info_get_name(pinfo),
+                             m_num_in_buses, m_queue);
+            
+            //m_buses_in[bus_slot]->init_in();
+            m_buses_in_active[bus_slot] = true;
+            m_buses_in_init[bus_slot] = true;
+
+            if ( !replacement ){            
+                m_num_in_buses++;
+            }
+        }	
+    }
+    
+    /* end loop for clients */
+    
+    
     /* midi input */
     /* poll descriptors */
 
@@ -954,14 +1321,14 @@ mastermidibus::port_start( int a_client, int a_port )
 
     /* allocate into */
     m_poll_descriptors = new pollfd[m_num_poll_descriptors];
-
+    
     /* get descriptors */
     snd_seq_poll_descriptors(m_alsa_seq,  
 			     m_poll_descriptors, 
 			     m_num_poll_descriptors, 
 			     POLLIN);
-
-	unlock();
+    
+    unlock();
 }
 
 void 
@@ -978,6 +1345,15 @@ mastermidibus::port_exit( int a_client, int a_port )
 		}
 	}
 
+        for( int i=0; i< m_num_in_buses; i++ ){
+
+		if( m_buses_in[i]->get_client() == a_client  &&
+			m_buses_in[i]->get_port() == a_port ){
+
+			m_buses_in_active[i] = false;
+		}
+	}
+
 	unlock();
 }
 
@@ -988,112 +1364,117 @@ mastermidibus::get_midi_event( event *a_in )
     lock();
     
     snd_seq_event_t *ev; 
-
+    
     bool sysex = false;
-  
+    
     /* temp for midi data */
     unsigned char buffer[0x1000];
-
+    
     snd_seq_event_input(m_alsa_seq, &ev);
+    
+    
+    
+    bool ret = false;
 
+    if (! global_manual_alsa_ports )
+    {
+        switch( ev->type ){ 
 
+            case SND_SEQ_EVENT_PORT_START:
+                {   
+                    //printf("SND_SEQ_EVENT_PORT_START:    addr[%d:%d]\n", 
+                    //	   ev->data.addr.client, ev->data.addr.port );
+                    port_start( ev->data.addr.client, ev->data.addr.port );
+                    ret = true; 
+                    break;
+                }
 
-	bool ret = false;
-	switch( ev->type ){ 
+            case SND_SEQ_EVENT_PORT_EXIT:     
+                {
+                    //printf("SND_SEQ_EVENT_PORT_EXIT:     addr[%d:%d]\n", 
+                    //	   ev->data.addr.client, ev->data.addr.port ); 
+                    port_exit( ev->data.addr.client, ev->data.addr.port );
+                    ret = true; 
+                    break;
+                }
 
-	case SND_SEQ_EVENT_PORT_START:
-		{   
-			//printf("SND_SEQ_EVENT_PORT_START:    addr[%d:%d]\n", 
-			//	   ev->data.addr.client, ev->data.addr.port );
-			port_start( ev->data.addr.client, ev->data.addr.port );
-			ret = true; 
-			break;
-		}
+            case SND_SEQ_EVENT_PORT_CHANGE:
+                {   
+                    //printf("SND_SEQ_EVENT_PORT_CHANGE:   addr[%d:%d]\n", 
+                    //	   ev->data.addr.client, 
+                    //	   ev->data.addr.port ); 
+                    ret = true; 
+                    break;
+                }
 
- 	case SND_SEQ_EVENT_PORT_EXIT:     
-		{
-			//printf("SND_SEQ_EVENT_PORT_EXIT:     addr[%d:%d]\n", 
-			//	   ev->data.addr.client, ev->data.addr.port ); 
-			port_exit( ev->data.addr.client, ev->data.addr.port );
-			ret = true; 
-			break;
-		}
- 
-	case SND_SEQ_EVENT_PORT_CHANGE:
-		{   
-			//printf("SND_SEQ_EVENT_PORT_CHANGE:   addr[%d:%d]\n", 
-			//	   ev->data.addr.client, 
-			//	   ev->data.addr.port ); 
-			ret = true; 
-			break;
-		}
-		
-	default: break;
-	 		
-	}
+            default: break;
 
-	if( ret ){
-		unlock();
-		return false;
-	}
+        }
+    }
+
+    if( ret ){
+        unlock();
+        return false;
+    }
     
     /* alsa midi parser */
     snd_midi_event_t *midi_ev;
     snd_midi_event_new( 0x1000, &midi_ev );
-
+    
     long bytes =
-      snd_midi_event_decode( midi_ev,
-			     buffer,
-			     0x1000,
-			     ev ); 
+        snd_midi_event_decode( midi_ev,
+                               buffer,
+                               0x1000,
+                               ev ); 
     
     a_in->set_timestamp( ev->time.tick );
     a_in->set_status( buffer[0] );
     a_in->set_size( bytes );
-
+    
     /* we will only get EVENT_SYSEX on the first 
        packet of midi data, the rest we have
        to poll for */
-    if ( buffer[0] == EVENT_SYSEX ){
-      
-      /* set up for sysex if needed */
-      a_in->start_sysex( );
-      sysex = a_in->append_sysex( buffer, bytes );
+    //if ( buffer[0] == EVENT_SYSEX ){
+    if ( 0 ){
+    
+        /* set up for sysex if needed */
+        a_in->start_sysex( );
+        sysex = a_in->append_sysex( buffer, bytes );
     }
     else {
-
-       a_in->set_data( buffer[1], buffer[2] );
-	   
-       // some keyboards send on's with vel 0 for off
-       if ( a_in->get_status() == EVENT_NOTE_ON &&
-			a_in->get_note_velocity() == 0x00 ){
-		   a_in->set_status( EVENT_NOTE_OFF );
-       }
-
-       sysex = false;
+        
+        a_in->set_data( buffer[1], buffer[2] );
+        
+        // some keyboards send on's with vel 0 for off
+        if ( a_in->get_status() == EVENT_NOTE_ON &&
+             a_in->get_note_velocity() == 0x00 ){
+            a_in->set_status( EVENT_NOTE_OFF );
+        }
+        
+        sysex = false;
     }
-
+    
     /* sysex messages might be more than one message */
     while ( sysex ){
-
-      snd_seq_event_input(m_alsa_seq, &ev);
-
-      bytes =
-	snd_midi_event_decode( midi_ev,
-			       buffer,
-			       0x1000,
-			       ev ); 
-      
-      sysex = a_in->append_sysex( buffer, bytes );
-  
+        
+        snd_seq_event_input(m_alsa_seq, &ev);
+        
+        bytes =
+            snd_midi_event_decode( midi_ev,
+                                   buffer,
+                                   0x1000,
+                                   ev ); 
+        
+        sysex = a_in->append_sysex( buffer, bytes );
+        
     }
- 
+    
     snd_seq_free_event( ev );
     snd_midi_event_free( midi_ev );
-
+    
     unlock();
-
-	return true;
+    
+    return true;
 }
 
 void 
