@@ -34,7 +34,7 @@ perform::perform()
     for (int i=0; i< c_max_sequence; i++ ){
 
 		m_seqs[i] = NULL;
-		m_seqs_active[i] = false;
+        m_seqs_active[i] = false;
 		
 		
     }
@@ -48,7 +48,7 @@ perform::perform()
     thread_trigger_width_ms = c_thread_trigger_width_ms;
 
     m_left_tick = 0;
-    m_right_tick = c_ppqn * 4;
+    m_right_tick = c_ppqn * 16;
     m_starting_tick = 0;
     
     midi_control zero = {false,false,0,0,0};
@@ -118,7 +118,11 @@ perform::perform()
     
 }
 
-
+void
+perform::init( void )
+{
+    m_master_bus.init( );
+}
 
 void
 perform::init_jack( void )
@@ -126,10 +130,12 @@ perform::init_jack( void )
 
 #ifdef JACK_SUPPORT
 
-    if ( global_with_jack_transport ){
+    if ( global_with_jack_transport  && !m_jack_running){
 
         m_jack_running = true;
         m_jack_master = true;
+
+        //printf ( "init_jack() m_jack_running[%d]\n", m_jack_running );
         
         do {
 
@@ -168,8 +174,42 @@ perform::init_jack( void )
             
         } while (0);
         
-    } else {
+    } 
+
+ 
+    
+#endif
+}
+
+
+
+
+
+
+
+
+void
+perform::deinit_jack( void )
+{
+
+#ifdef JACK_SUPPORT
+
+    if ( global_with_jack_transport  && m_jack_running){
+
+         //printf ( "deinit_jack() m_jack_running[%d]\n", m_jack_running );
+        
         m_jack_running = false;
+        m_jack_master = false;
+
+        if ( jack_release_timebase(m_jack_client)){
+            printf("Cannot release Timebase.\n");
+        }
+        
+        if (jack_client_close(m_jack_client)) {
+            printf("Cannot close JACK client.\n");
+        }
+            
+
     }
 
     if ( !m_jack_running ){
@@ -180,6 +220,8 @@ perform::init_jack( void )
 }
 
 
+
+
 void
 perform::clear_all( void )
 {
@@ -188,7 +230,7 @@ perform::clear_all( void )
 
     for (int i=0; i< c_max_sequence; i++ ){
 
-        if ( m_seqs_active[i] )
+        if ( is_active(i) )
             delete_sequence( i );
     }
     
@@ -202,6 +244,18 @@ perform::clear_all( void )
 
 
 
+void
+perform::mute_all_tracks( void )
+{
+    for (int i=0; i< c_max_sequence; i++ )
+    {    
+        if ( is_active(i) )
+            m_seqs[i]->set_song_mute( true );
+              
+    }
+}
+
+
 
 
 perform::~perform()
@@ -219,7 +273,7 @@ perform::~perform()
         pthread_join( m_in_thread, NULL );
 
     for (int i=0; i< c_max_sequence; i++ ){
-	if ( m_seqs_active[i] == true ){
+	if ( is_active(i) ){
 	    delete m_seqs[i];
 	}
     }
@@ -233,7 +287,7 @@ perform::set_left_tick( long a_tick )
  
     if ( m_left_tick >= m_right_tick )
 	m_right_tick = m_left_tick + c_ppqn * 4;
- 
+    
 }
 
 long 
@@ -281,21 +335,21 @@ perform::add_sequence( sequence *a_seq, int a_perf )
 {
     /* check for perferred */
     if ( a_perf < c_max_sequence &&  
-	 m_seqs_active[a_perf] == false &&
+	 is_active(a_perf) == false &&
 	 a_perf >= 0 ){
 		    
       m_seqs[a_perf] = a_seq;
-      m_seqs_active[a_perf] = true;
+      set_active(a_perf, true);
 	    //a_seq->set_tag( a_perf );
 
     } else {
 
 	for (int i=a_perf; i< c_max_sequence; i++ ){
 	    
-	    if ( m_seqs_active[i] == false ){
+	    if ( is_active(i) == false ){
 
 	      m_seqs[i] = a_seq;
-	      m_seqs_active[i] = true;
+	      set_active(i,true);
 
 		//a_seq->set_tag( i  );
 		break;
@@ -305,15 +359,122 @@ perform::add_sequence( sequence *a_seq, int a_perf )
 }
 
 
+void
+perform::set_active( int a_sequence, bool a_active )
+{
+    if ( a_sequence < 0 || a_sequence >= c_max_sequence )
+        return;
+
+    //printf ("set_active %d\n", a_active );
+    
+    if ( m_seqs_active[ a_sequence ] == true && a_active == false )
+    {
+        set_was_active(a_sequence);
+    }
+    
+    m_seqs_active[ a_sequence ] = a_active;
+
+    
+}
+    
+
+void 
+perform::set_was_active( int a_sequence )
+{
+    
+    if ( a_sequence < 0 || a_sequence >= c_max_sequence )
+        return;
+    
+        //printf( "was_active true\n" );
+        
+        m_was_active_main[ a_sequence ] = true;
+        m_was_active_edit[ a_sequence ] = true;
+        m_was_active_perf[ a_sequence ] = true;
+        m_was_active_names[ a_sequence ] = true;
+}
+ 
+
+
 bool 
 perform::is_active( int a_sequence )
 {
     if ( a_sequence < 0 || a_sequence >= c_max_sequence )
-	return false;
+	    return false;
     
     return m_seqs_active[ a_sequence ];
 }
 
+
+bool 
+perform::is_dirty_main (int a_sequence)
+{
+    if ( a_sequence < 0 || a_sequence >= c_max_sequence )
+	   return false;
+
+    if ( is_active(a_sequence) )
+    {
+        return m_seqs[a_sequence]->is_dirty_main();
+    }
+
+    bool was_active = m_was_active_main[ a_sequence ];
+    m_was_active_main[ a_sequence ] = false;
+    
+    return was_active;
+}
+
+
+bool 
+perform::is_dirty_edit (int a_sequence)
+{
+    if ( a_sequence < 0 || a_sequence >= c_max_sequence )
+	    return false;
+
+    if ( is_active(a_sequence) )
+    {
+        return m_seqs[a_sequence]->is_dirty_edit();
+    }
+    
+    bool was_active = m_was_active_edit[ a_sequence ];
+    m_was_active_edit[ a_sequence ] = false;
+    
+    return was_active;
+}
+
+
+bool 
+perform::is_dirty_perf (int a_sequence)
+{
+    if ( a_sequence < 0 || a_sequence >= c_max_sequence )
+	return false;
+
+    if ( is_active(a_sequence) )
+    {
+        return m_seqs[a_sequence]->is_dirty_perf();
+    }
+
+    bool was_active = m_was_active_perf[ a_sequence ];
+    m_was_active_perf[ a_sequence ] = false;
+    
+    return was_active;
+}
+
+bool 
+perform::is_dirty_names (int a_sequence)
+{
+    if ( a_sequence < 0 || a_sequence >= c_max_sequence )
+	return false;
+
+    if ( is_active(a_sequence) )
+    {
+        return m_seqs[a_sequence]->is_dirty_names();
+    } 
+ 
+    bool was_active = m_was_active_names[ a_sequence ];
+    m_was_active_names[ a_sequence ] = false;
+    
+    return was_active;
+}
+
 sequence* 
 perform::get_sequence( int a_sequence )
 {
@@ -362,7 +523,7 @@ perform::get_bpm( )
 void
 perform::delete_sequence( int a_num )
 {
-	m_seqs_active[a_num] = false;
+	set_active(a_num, false);
   
     if ( m_seqs[a_num] != NULL &&
          !m_seqs[a_num]->get_editing() ){
@@ -387,14 +548,14 @@ perform::new_sequence( int a_sequence )
 
     m_seqs[ a_sequence ] = new sequence();
     m_seqs[ a_sequence ]->set_master_midi_bus( &m_master_bus );
-    m_seqs_active[ a_sequence ] = true;
+    set_active(a_sequence, true);
 
 }
 
 midi_control *
 perform::get_midi_control_toggle( unsigned int a_seq )
 {
-	if ( a_seq >= c_midi_controls )
+	if ( a_seq >= (unsigned int) c_midi_controls )
 		return NULL;
 	return &m_midi_cc_toggle[a_seq];
 
@@ -403,7 +564,7 @@ perform::get_midi_control_toggle( unsigned int a_seq )
 midi_control *
 perform::get_midi_control_on( unsigned int a_seq )
 {
-	if ( a_seq >= c_midi_controls )
+	if ( a_seq >= (unsigned int) c_midi_controls )
 		return NULL;
 	return &m_midi_cc_on[a_seq];
 }
@@ -411,7 +572,7 @@ perform::get_midi_control_on( unsigned int a_seq )
 midi_control *
 perform::get_midi_control_off( unsigned int a_seq )
 {
-	if ( a_seq >= c_midi_controls )
+	if ( a_seq >= (unsigned int) c_midi_controls )
 		return NULL;
 	return &m_midi_cc_off[a_seq];
 }
@@ -483,7 +644,7 @@ perform::play( long a_tick )
     m_tick = a_tick;	
     for (int i=0; i< c_max_sequence; i++ ){
 		
-		if ( m_seqs_active[i] == true ){
+		if ( is_active(i) ){
 			assert( m_seqs[i] );
 			
 			
@@ -505,12 +666,9 @@ perform::play( long a_tick )
 void 
 perform::set_orig_ticks( long a_tick  )
 {
-
-    //printf( "set_orig_ticks() [%d]\n", a_tick );
-    
     for (int i=0; i< c_max_sequence; i++ ){
 	
-	if ( m_seqs_active[i] == true ){
+	if ( is_active(i) == true ){
 	    assert( m_seqs[i] );
 	    m_seqs[i]->set_orig_tick( a_tick );
 	} 
@@ -520,7 +678,7 @@ perform::set_orig_ticks( long a_tick  )
 void 
 perform::clear_sequence_triggers( int a_seq  )
 {
-	if ( m_seqs_active[a_seq] == true ){
+	if ( is_active(a_seq) == true ){
 		assert( m_seqs[a_seq] );
 		m_seqs[a_seq]->clear_triggers( );
 	} 
@@ -535,7 +693,7 @@ perform::move_triggers( bool a_direction )
 
 	for (int i=0; i< c_max_sequence; i++ ){
 
-	    if ( m_seqs_active[i] == true ){
+	    if ( is_active(i) == true ){
 		assert( m_seqs[i] );
 		m_seqs[i]->move_triggers( m_left_tick, distance, a_direction );
 	    } 
@@ -543,6 +701,30 @@ perform::move_triggers( bool a_direction )
     }
 }
 
+void
+perform::push_trigger_undo( void )
+{
+    for (int i=0; i< c_max_sequence; i++ ){
+        
+        if ( is_active(i) == true ){
+            assert( m_seqs[i] );
+            m_seqs[i]->push_trigger_undo( );
+        } 
+    }
+}
+
+void
+perform::pop_trigger_undo( void )
+{
+    for (int i=0; i< c_max_sequence; i++ ){
+        
+        if ( is_active(i) == true ){
+            assert( m_seqs[i] );
+            m_seqs[i]->pop_trigger_undo( );
+        } 
+    }
+}
+
 
 /* copies between L and R -> R */
 void
@@ -554,7 +736,7 @@ perform::copy_triggers( )
 
 	for (int i=0; i< c_max_sequence; i++ ){
 
-	    if ( m_seqs_active[i] == true ){
+	    if ( is_active(i) == true ){
 		assert( m_seqs[i] );
 		m_seqs[i]->copy_triggers( m_left_tick, distance );
 	    } 
@@ -625,10 +807,10 @@ perform::position_jack( bool a_state )
      * or time signature changes at specific locations in the
      * transport timeline. */
     
-    current_tick *= 10.0;
+    current_tick *= 10;
     
-    pos.bar  =  current_tick / (long) pos.ticks_per_beat / pos.beats_per_bar;
-    pos.beat = (current_tick / (long) pos.ticks_per_beat) % 4; 
+    pos.bar  = (int) current_tick / (long) pos.ticks_per_beat / pos.beats_per_bar;
+    pos.beat = (int) (current_tick / (long) pos.ticks_per_beat) % 4; 
     pos.tick = current_tick % (c_ppqn * 10);
 
     pos.bar_start_tick = pos.bar * pos.beats_per_bar * pos.ticks_per_beat;
@@ -721,7 +903,7 @@ perform::off_sequences( void )
 {
     for (int i=0; i< c_max_sequence; i++ ){
 		
-		if ( m_seqs_active[i] == true ){
+		if ( is_active(i) == true ){
 			assert( m_seqs[i] );
 			m_seqs[i]->set_playing( false );
 			
@@ -737,7 +919,7 @@ perform::reset_sequences( void )
 {
     for (int i=0; i< c_max_sequence; i++ ){
 		
-		if ( m_seqs_active[i] == true ){
+		if ( is_active(i) == true ){
 			assert( m_seqs[i] );
 
                         bool state = m_seqs[i]->get_playing();
@@ -793,7 +975,7 @@ perform::get_max_trigger( void )
 
     for (int i=0; i< c_max_sequence; i++ ){
 	
-	if ( m_seqs_active[i] == true ){
+	if ( is_active(i) == true ){
 	    assert( m_seqs[i] );
 	    
 	    t = m_seqs[i]->get_max_trigger( );  
@@ -844,7 +1026,7 @@ output_thread_func(void *a_pef )
 int jack_sync_callback(jack_transport_state_t state, 
 					   jack_position_t *pos, void *arg)
 {
-    //printf( "jack_sync_callback() " );
+  //printf( "jack_sync_callback() " );
 
   perform *p = (perform *) arg;
 
@@ -900,15 +1082,15 @@ int jack_sync_callback(jack_transport_state_t state,
 #endif
 
 
-void
+    void
 perform::output_func(void)
 {
 
-    
+
     while(m_outputing){ 
 
         //printf ("waiting for signal\n");
-        
+
         m_condition_var.lock();
 
         while ( !m_running ){
@@ -924,442 +1106,434 @@ perform::output_func(void)
 
         //printf( "signaled [%d]\n", m_playback_mode ); 
 
-        
+
 
         /* begning time */
         struct timespec last;
         /* current time */
         struct timespec current;
-        
+
         struct timespec stats_loop_start;
         struct timespec stats_loop_finish;
-        
-        
+
+
         /* difference between last and current */
         struct timespec delta;
-        
+
         /* tick and tick fraction */
-        long current_tick   = 0;
-        long current_tick_f = 0;
-        
-        long total_tick   = 0;
-        long total_tick_f = 0;
-        
+        double current_tick   = 0.0;
+        double total_tick   = 0.0;
+        double clock_tick = 0.0;
+
         long stats_total_tick = 0;
-        
+
         long stats_loop_index = 0;
         long stats_min = 0x7FFFFFFF;
         long stats_max = 0;
         long stats_avg = 0;
         long stats_last_clock_us = 0;
         long stats_clock_width_us = 0;
-        
+
         long stats_all[100];
         long stats_clock[100];
 
         bool jack_stopped = false;
         bool dumping = false;
 
-              
+        bool init_clock = true;
+
+        double jack_ticks_converted = 0.0;
+        double jack_ticks_converted_last = 0.0;
+        double jack_ticks_delta = 0.0;
 
-        
         for( int i=0; i<100; i++ ){
             stats_all[i] = 0;
             stats_clock[i] = 0;
         }
-        
+
         /* if we are in the performance view, we care 
            about starting from the offset */
-        if ( m_playback_mode ){
+        if ( m_playback_mode && !m_jack_running){
 
             current_tick = m_starting_tick;
+            clock_tick = m_starting_tick;
             set_orig_ticks( m_starting_tick ); 
-            
+
         }
 
 
         int ppqn = m_master_bus.get_ppqn();
-        
-        m_master_bus.start();
-        
         /* get start time position */
         clock_gettime(CLOCK_REALTIME, &last);
-        
+
         if ( global_stats )
             stats_last_clock_us= (last.tv_sec * 1000000) + (last.tv_nsec / 1000);
-        
-        
-        
-        
+
+
+
+
         while( m_running ){
-            
+
             /************************************
 
-	        Get delta time ( current - last )
-	        Get delta ticks from time
-	        Add to current_ticks
-	        Compute prebuffer ticks
-	        play from current tick to prebuffer
+              Get delta time ( current - last )
+              Get delta ticks from time
+              Add to current_ticks
+              Compute prebuffer ticks
+              play from current tick to prebuffer
 
-            **************************************/
+             **************************************/
 
             if ( global_stats ){
                 clock_gettime(CLOCK_REALTIME, &stats_loop_start);
             }
-            
-            
+
+
             /* delta time */
             clock_gettime(CLOCK_REALTIME, &current);
             delta.tv_sec  =  (current.tv_sec  - last.tv_sec  );
             delta.tv_nsec =  (current.tv_nsec - last.tv_nsec );
             long delta_us = (delta.tv_sec * 1000000) + (delta.tv_nsec / 1000);
 
-            
+
             /* delta time to ticks */
             /* bpm */
             int bpm  = m_master_bus.get_bpm();
-            
+
             /* get delta ticks, delta_ticks_f is in 1000th of a tick */
-            long delta_tick   =  (long) (bpm * ppqn * (delta_us/60000000.0f) ); 
-            long delta_tick_f = ((long) (bpm * ppqn * (delta_us/60000.0f) )) % 1000; 
-            
+            double delta_tick   =  (double) (bpm * ppqn * (delta_us/60000000.0f) ); 
+
             //printf ( "delta_tick[%ld.%03ld]\n", delta_tick, delta_tick_f  );
-            
-            /* add delta to current ticks */
-            current_tick   += delta_tick;
-            current_tick_f += delta_tick_f;
-            current_tick   += ( current_tick_f / 1000 );
-            current_tick_f %= 1000;
-            
-            total_tick   += delta_tick;
-            total_tick_f += delta_tick_f;
-            total_tick   += (total_tick_f / 1000 );
-            total_tick_f %= 1000;
-
-
-            
-            
 #ifdef JACK_SUPPORT
+
+            // no init until we get a good lock
             
             if ( m_jack_running ){
-
                
+                init_clock = false;
 
                 m_jack_transport_state = jack_transport_query( m_jack_client, &m_jack_pos );
                 m_jack_frame_current =  jack_get_current_transport_frame( m_jack_client );
-
-
-//               switch (  m_jack_transport_state_last   ){
-                    
-//                     case JackTransportStopped:
-//                         printf( "[Stopped ] " ); break;
-//                     case JackTransportRolling:
-//                         printf( "[Rolling ] " ); break;
-//                     case JackTransportStarting:
-//                         printf( "[Starting] " ); break;
-//                 }
-                
-                
-//                 switch (  m_jack_transport_state   ){
-                    
-//                     case JackTransportStopped:
-//                         printf( "[Stopped ]\n" ); break;
-//                     case JackTransportRolling:
-//                         printf( "[Rolling ]\n" ); break;
-//                     case JackTransportStarting:
-//                         printf( "[Starting]\n" ); break;
-//                 }
-
-
-                
+ 
                 if ( m_jack_transport_state_last  ==  JackTransportStarting &&
-                     m_jack_transport_state        == JackTransportRolling ){
+                     m_jack_transport_state       == JackTransportRolling ){
 
                     m_jack_frame_last = m_jack_frame_current;
 
-                    
-                    //printf ("[Start Playback]\n" );
+
+                    printf ("[Start Playback]\n" );
                     dumping = true;
-
-                     
-
                     m_jack_tick =
                         m_jack_pos.frame *
                         m_jack_pos.ticks_per_beat *
                         m_jack_pos.beats_per_minute / (m_jack_pos.frame_rate * 60.0);
 
-                    
-                    /* convert ticks */
-                    double jack_tick_converted =
-                        m_jack_tick * ((double) c_ppqn /
-                                          (m_jack_pos.ticks_per_beat *
-                                           m_jack_pos.beat_type / 4.0  ));
-                    
-                    set_orig_ticks( jack_tick_converted );
-                    
 
+                    /* convert ticks */
+                    jack_ticks_converted =
+                        m_jack_tick * ((double) c_ppqn /
+                                (m_jack_pos.ticks_per_beat *
+                                 m_jack_pos.beat_type / 4.0  ));
+
+                    set_orig_ticks( (long) jack_ticks_converted );
+                    current_tick = clock_tick = total_tick = jack_ticks_converted_last = jack_ticks_converted;
+                    init_clock = true;
+
+                    if ( m_looping && m_playback_mode ){
+
+                        //printf( "left[%lf] right[%lf]\n", (double) get_left_tick(), (double) get_right_tick() );
+                        
+                        if ( current_tick >= get_right_tick() ){
+
+                            while ( current_tick >= get_right_tick() ){
+
+                                double size = get_right_tick() - get_left_tick();
+                                current_tick = current_tick - size;
+                                
+                                //printf( "> current_tick[%lf]\n", current_tick );
+                            }        
+                            reset_sequences();
+                            set_orig_ticks( current_tick );
+                        }
+                    }
                 }
 
                 if ( m_jack_transport_state_last  ==  JackTransportRolling &&
-                     m_jack_transport_state  == JackTransportStopped ){
+                        m_jack_transport_state  == JackTransportStopped ){
 
-                    
                     m_jack_transport_state_last = JackTransportStopped;
                     //printf ("[Stop Playback]\n" );
                     jack_stopped = true;
                 }
 
-
-                
-                
-
                 //-----  Jack transport is Rolling Now ---------
 
-
-                
-                m_jack_frame_current =  jack_get_current_transport_frame( m_jack_client );
-
-
-                //printf( " frame[%7d]", m_jack_pos.frame );
-                //printf( " current_transport_frame[%7d]", m_jack_frame_current );
-                
-                // if we are moving ahead
-                if ( (m_jack_frame_current > m_jack_frame_last)){
-                    
-                     
-                    m_jack_tick +=
-                        (m_jack_frame_current - m_jack_frame_last)  *
-                        m_jack_pos.ticks_per_beat *
-                        m_jack_pos.beats_per_minute / (m_jack_pos.frame_rate * 60.0);
-                    
-                    m_jack_frame_last = m_jack_frame_current;
-
-                    //printf( " | " );
-                    
-                }
-                else
+                /* transport is in a sane state if dumping == true */
+                if ( dumping )
                 {
-                    //printf( " * " );
-                }
+                    m_jack_frame_current =  jack_get_current_transport_frame( m_jack_client );
 
-                /* convert ticks */
-                double jack_tick_converted =
-                    m_jack_tick * ((double) c_ppqn /
-                                   (m_jack_pos.ticks_per_beat * m_jack_pos.beat_type / 4.0  ));
+                    //printf( " frame[%7d]", m_jack_pos.frame );
+                    //printf( " current_transport_frame[%7d]", m_jack_frame_current );
 
-                current_tick   = jack_tick_converted;
-                current_tick_f = 0;
-                total_tick     = jack_tick_converted;
-                total_tick_f   = 0;
-
-                m_jack_transport_state_last = m_jack_transport_state;
+                    // if we are moving ahead
+                    if ( (m_jack_frame_current > m_jack_frame_last)){
 
 
-                // PRINTS
-//               long ptick, pbeat, pbar;
+                        m_jack_tick +=
+                            (m_jack_frame_current - m_jack_frame_last)  *
+                            m_jack_pos.ticks_per_beat *
+                            m_jack_pos.beats_per_minute / (m_jack_pos.frame_rate * 60.0);
 
-//                 pbar  = (long) ((long) m_jack_tick / (m_jack_pos.ticks_per_beat *  m_jack_pos.beats_per_bar ));
-      
-//                 pbeat = (long) ((long) m_jack_tick % (long) (m_jack_pos.ticks_per_beat *  m_jack_pos.beats_per_bar ));
-//                 pbeat = pbeat / (long) m_jack_pos.ticks_per_beat;
-                
-//                 ptick = (long) m_jack_tick % (long) m_jack_pos.ticks_per_beat;
-                
-                
-//                 printf( " bbb [%2d:%2d:%4d]", pbar+1, pbeat+1, ptick );
-//                 printf( " bbb [%2d:%2d:%4d]", m_jack_pos.bar, m_jack_pos.beat, m_jack_pos.tick );
-                
-//                 double jack_tick = (m_jack_pos.bar-1) * (m_jack_pos.ticks_per_beat *  m_jack_pos.beats_per_bar ) +
-//                     (m_jack_pos.beat-1) * m_jack_pos.ticks_per_beat + m_jack_pos.tick;
-                
-//                 printf( " jtick[%8.3f]", m_jack_tick );
-//                 printf( " mtick[%8.3f]", jack_tick );
-                
-//                 printf( " delta[%8.3f]", m_jack_tick - jack_tick );
-                
-//                 printf( "\n");
-                
 
-            } else {
+                        //printf ( "m_jack_tick += (m_jack_frame_current[%lf] - m_jack_frame_last[%lf]) *\n",
+                        //        (double) m_jack_frame_current, (double) m_jack_frame_last );
+                        //printf(  "m_jack_pos.ticks_per_beat[%lf] * m_jack_pos.beats_per_minute[%lf] / \n(m_jack_pos.frame_rate[%lf] * 60.0\n", (double) m_jack_pos.ticks_per_beat, (double) m_jack_pos.beats_per_minute, (double) m_jack_pos.frame_rate);
+           
+                        
+                        m_jack_frame_last = m_jack_frame_current;
+                    }
+
+                    /* convert ticks */
+                    jack_ticks_converted =
+                        m_jack_tick * ((double) c_ppqn /
+                                (m_jack_pos.ticks_per_beat * m_jack_pos.beat_type / 4.0  ));
+
+                    //printf ( "jack_ticks_conv[%lf] = \n",  jack_ticks_converted ); 
+                    //printf ( "    m_jack_tick[%lf] * ((double) c_ppqn[%lf] / \n", m_jack_tick, (double) c_ppqn );
+                    //printf ( "   (m_jack_pos.ticks_per_beat[%lf] * m_jack_pos.beat_type[%lf] / 4.0  )\n",
+                    //        m_jack_pos.ticks_per_beat, m_jack_pos.beat_type );
+                            
+                          
+
+          
+                    jack_ticks_delta = jack_ticks_converted - jack_ticks_converted_last;
+
+                    clock_tick     += jack_ticks_delta;
+                    current_tick   += jack_ticks_delta;
+                    total_tick     += jack_ticks_delta;
+
+                    m_jack_transport_state_last = m_jack_transport_state;
+                    jack_ticks_converted_last = jack_ticks_converted;
+
+                    /* printf( "current_tick[%lf] delta[%lf]\n", current_tick, jack_ticks_delta ); */
+
+
+                    long ptick, pbeat, pbar;
+
+                    pbar  = (long) ((long) m_jack_tick / (m_jack_pos.ticks_per_beat *  m_jack_pos.beats_per_bar ));
+
+                    pbeat = (long) ((long) m_jack_tick % (long) (m_jack_pos.ticks_per_beat *  m_jack_pos.beats_per_bar ));
+                    pbeat = pbeat / (long) m_jack_pos.ticks_per_beat;
+
+                    ptick = (long) m_jack_tick % (long) m_jack_pos.ticks_per_beat;
+
+
+                    //printf( " bbb [%2d:%2d:%4d]", pbar+1, pbeat+1, ptick );
+                    //printf( " bbb [%2d:%2d:%4d]", m_jack_pos.bar, m_jack_pos.beat, m_jack_pos.tick );
+
+                    double jack_tick = (m_jack_pos.bar-1) * (m_jack_pos.ticks_per_beat *  m_jack_pos.beats_per_bar ) +
+                        (m_jack_pos.beat-1) * m_jack_pos.ticks_per_beat + m_jack_pos.tick;
+
+                    //printf( " jtick[%8.3f]", m_jack_tick );
+                    //printf( " mtick[%8.3f]", jack_tick );
+
+                    //printf( " delta[%8.3f]", m_jack_tick - jack_tick );
+
+                    //printf( "\n");
+
+                } /* end if dumping / sane state */
+
+            } /* if jack running */
+            else 
+            {
+#endif
+                /* default if jack is not compiled in, or not running */
+                /* add delta to current ticks */
+                clock_tick     += delta_tick;
+                current_tick   += delta_tick;
+                total_tick     += delta_tick;
                 dumping = true;
+
+#ifdef JACK_SUPPORT
+            }
+#endif
+
+            /* init_clock will be true when we run for the first time, or
+             * as soon as jack gets a good lock on playback */
+            
+            if( init_clock ) 
+            {
+                m_master_bus.init_clock( clock_tick );
+                init_clock = false;
             }
             
-#else
-            dumping = true;
-#endif
-            if ( dumping ){
-                
-                if ( m_looping && m_playback_mode && !m_jack_running ){
-                    
-                    if ( current_tick >= get_right_tick() ){
-                        
-                        long leftover_tick = current_tick - (get_right_tick());
-                        
+            if ( dumping )
+            {
+                if ( m_looping && m_playback_mode )
+                {
+                    if ( current_tick >= get_right_tick() )
+                    {
+                        double leftover_tick = current_tick - (get_right_tick());
+
                         play( get_right_tick() - 1 );
                         reset_sequences();
-                    
+
                         set_orig_ticks( get_left_tick() );
-                        current_tick = get_left_tick() + leftover_tick;
+                        current_tick = (double) get_left_tick() + leftover_tick;
                     }
                 }
-                
-                
-                
-                
+
                 /* play */
-                play( current_tick );
+                play( (long) current_tick );
                 //printf( "play[%d]\n", current_tick );
-                
+
                 /* midi clock */
-                m_master_bus.clock( total_tick );
-                
-                
+                m_master_bus.clock( (long) clock_tick );
+
+
                 if ( global_stats ){	  
-                    
+
                     while ( stats_total_tick <= total_tick ){
-                        
+
                         /* was there a tick ? */
                         if ( stats_total_tick % (c_ppqn / 24) == 0 ){
-                            
+
                             long current_us = (current.tv_sec * 1000000) + (current.tv_nsec / 1000);
                             stats_clock_width_us = current_us - stats_last_clock_us;
                             stats_last_clock_us = current_us;
-                            
+
                             int index = stats_clock_width_us / 300;
                             if ( index >= 100 ) index = 99; 
                             stats_clock[index]++;
-                            
+
                         }
                         stats_total_tick++;
                     }
-                    
                 }
             }
 
             /***********************************
-         
+
               Figure out how much time 
               we need to sleep, and do it
 
-            ************************************/
+             ************************************/
 
             /* set last */
             last = current;
-            
+
             clock_gettime(CLOCK_REALTIME, &current);
             delta.tv_sec  =  (current.tv_sec  - last.tv_sec  );
             delta.tv_nsec =  (current.tv_nsec - last.tv_nsec );
             long elapsed_us = (delta.tv_sec * 1000000) + (delta.tv_nsec / 1000);
             //printf( "elapsed_us[%ld]\n", elapsed_us );
-            
+
             /* now, we want to trigger every c_thread_trigger_width_ms,
                and it took us delta_us to play() */ 
-            
+
             delta_us = (c_thread_trigger_width_ms * 1000) - elapsed_us;
             //printf( "sleeping_us[%ld]\n", delta_us );
-            
-            
+
+
             /* check midi clock adjustment */
-            
-            long next_total_tick = (total_tick + (c_ppqn / 24)) - (total_tick % (c_ppqn / 24)); 
-            
-            long next_clock_delta   = (next_total_tick - total_tick - 1); 
-            long next_clock_delta_f = (1000 - total_tick_f);
-            
-            long next_clock_delta_us =  (( next_clock_delta ) * 60000000 / c_ppqn  / bpm );
-            next_clock_delta_us += (( next_clock_delta_f ) * 60000 / c_ppqn  / bpm );
-            
-            if ( next_clock_delta_us < (c_thread_trigger_width_ms * 1000 * 2) ){
+
+            double next_total_tick = (total_tick + (c_ppqn / 24.0)); 
+            double next_clock_delta   = (next_total_tick - total_tick - 1); 
+
+
+            double next_clock_delta_us =  (( next_clock_delta ) * 60000000.0f / c_ppqn  / bpm );
+
+            if ( next_clock_delta_us < (c_thread_trigger_width_ms * 1000.0f * 2.0f) ){
                 delta_us = next_clock_delta_us;
             } 
-            
-            
-            if ( delta_us > 0 ){
-                
+
+
+            if ( delta_us > 0.0 ){
+
                 delta.tv_sec =  (delta_us / 1000000);
                 delta.tv_nsec = (delta_us % 1000000) * 1000;
-                
+
                 //printf("sleeping() ");
                 nanosleep( &delta, NULL );
-                
+
             } 
-            
+
             else {
-                
-                printf ("underrun\n" );
-                //pthread_exit(0);  
-                
+
+                if ( global_stats )
+                    printf ("underrun\n" );
             }
-            
+
             if ( global_stats ){	  
                 clock_gettime(CLOCK_REALTIME, &stats_loop_finish);
             }
-            
+
             if ( global_stats ){
-                
+
                 delta.tv_sec  =  (stats_loop_finish.tv_sec  - stats_loop_start.tv_sec  );
                 delta.tv_nsec =  (stats_loop_finish.tv_nsec - stats_loop_start.tv_nsec );
                 long delta_us = (delta.tv_sec * 1000000) + (delta.tv_nsec / 1000);
-                
+
                 int index = delta_us / 100;
                 if ( index >= 100  ) index = 99;
-                
+
                 stats_all[index]++;
-                
+
                 if ( delta_us > stats_max )
                     stats_max = delta_us;
                 if ( delta_us < stats_min )
                     stats_min = delta_us;
-                
+
                 stats_avg += delta_us;
-                
+
                 stats_loop_index++;	  
-                
+
                 if ( stats_loop_index > 200 ){
-                    
+
                     stats_loop_index = 0;
                     stats_avg /= 200;
-                    
+
                     printf( "stats_avg[%d]us stats_min[%d]us stats_max[%d]us\n", stats_avg, stats_min, stats_max );
-                    
+
                     stats_min = 0x7FFFFFFF;
                     stats_max = 0;
                     stats_avg = 0;
-                    
+
                 }
-                
+
             }
 
             if (jack_stopped )
                 inner_stop();
         }
-        
-        
+
+
         if ( global_stats ){
-            
+
             printf ( "\n\n-- trigger width --\n" );
             for ( int i=0; i<100; i++ ){
                 printf( "[%3d][%8d]\n", i * 100, stats_all[i] );
             }
             printf ( "\n\n-- clock width --\n" );
             int bpm  = m_master_bus.get_bpm();
-            
+
             printf ( "optimal : [%ld]us\n", ((c_ppqn / 24)* 60000000 / c_ppqn  / bpm )); 
-            
-            
+
+
             for ( int i=0; i<100; i++ ){
                 printf( "[%3d][%8d]\n", i * 300, stats_clock[i] );
             }
-            
-            
+
+
         }
-        
+
         m_tick = 0;
         m_master_bus.flush( );
         m_master_bus.stop();
 
     }
-    
+
     pthread_exit(0);
 }
 
@@ -1405,28 +1579,28 @@ perform::handle_midi_control( int a_control, bool a_state )
     switch( a_control ){
 
         case c_midi_control_bpm_up:
-            printf ( "bpm up\n" );
+            //printf ( "bpm up\n" );
             set_bpm( get_bpm() + 1 );   
             break;
             
         case c_midi_control_bpm_dn:
-            printf ( "bpm dn\n" );
+            //printf ( "bpm dn\n" );
             set_bpm( get_bpm() - 1 );
             break;
             
         case c_midi_control_ss_up:
-            printf ( "ss up\n" );
+            //printf ( "ss up\n" );
             set_screenset( get_screenset() + 1 );  
             break;
             
         case c_midi_control_ss_dn:
-            printf ( "ss dn\n" );
+            //printf ( "ss dn\n" );
             set_screenset( get_screenset() - 1 );
             break;
             
         case c_midi_control_mod_replace:
 
-            printf ( "replace\n" );
+            //printf ( "replace\n" );
 
             
             if ( a_state )
@@ -1438,7 +1612,7 @@ perform::handle_midi_control( int a_control, bool a_state )
             
         case c_midi_control_mod_snapshot:
 
-            printf ( "snapshot\n" );
+            //printf ( "snapshot\n" );
 
             if ( a_state )
                 set_sequence_control_status( c_status_snapshot );
@@ -1449,7 +1623,7 @@ perform::handle_midi_control( int a_control, bool a_state )
             
         case c_midi_control_mod_queue:
 
-            printf ( "queue\n" );
+            //printf ( "queue\n" );
 
             
             if ( a_state )
@@ -1591,7 +1765,7 @@ perform::save_playing_state( void )
 {
 	for( int i=0; i<c_total_seqs; i++ ){
 		
-		if ( m_seqs_active[i] == true ){
+		if ( is_active(i) == true ){
 			assert( m_seqs[i] );
 			m_sequence_state[i] = m_seqs[i]->get_playing();
 		}
@@ -1605,7 +1779,7 @@ perform::restore_playing_state( void )
 {
 	for( int i=0; i<c_total_seqs; i++ ){
 		
-		if ( m_seqs_active[i] == true ){
+		if ( is_active(i) == true ){
 			assert( m_seqs[i] );
 			m_seqs[i]->set_playing( m_sequence_state[i] );
 		}
@@ -1639,7 +1813,7 @@ perform::unset_sequence_control_status( int a_status )
 void
 perform::sequence_playing_toggle( int a_sequence )
 {
-    if ( m_seqs_active[a_sequence] == true ){
+    if ( is_active(a_sequence) == true ){
 		assert( m_seqs[a_sequence] );
 
 		if ( m_control_status & c_status_queue ){
@@ -1662,7 +1836,7 @@ perform::sequence_playing_toggle( int a_sequence )
 void
 perform::sequence_playing_on( int a_sequence )
 {
-    if ( m_seqs_active[a_sequence] == true ){
+    if ( is_active(a_sequence) == true ){
 		assert( m_seqs[a_sequence] );
 		m_seqs[a_sequence]->set_playing(true);
     } 
@@ -1672,7 +1846,7 @@ perform::sequence_playing_on( int a_sequence )
 void
 perform::sequence_playing_off( int a_sequence )
 {
-    if ( m_seqs_active[a_sequence] == true ){
+    if ( is_active(a_sequence) == true ){
 		assert( m_seqs[a_sequence] );
 		m_seqs[a_sequence]->set_playing(false);
     } 
