@@ -44,7 +44,7 @@
 
 
 using namespace Gtk;
-using SigC::slot;
+
 
 /* performance roll */
 class perfroll : public Gtk::DrawingArea
@@ -54,7 +54,7 @@ class perfroll : public Gtk::DrawingArea
 
     Glib::RefPtr<Gdk::GC> m_gc;
     Glib::RefPtr<Gdk::Window> m_window;
-    Gdk::Color    m_black, m_white, m_grey;
+    Gdk::Color    m_black, m_white, m_grey, m_lt_grey;
 
     Glib::RefPtr<Gdk::Pixmap> m_pixmap;
     Glib::RefPtr<Gdk::Pixmap> m_background;
@@ -63,6 +63,8 @@ class perfroll : public Gtk::DrawingArea
     perform        *m_mainperf;
 
     int          m_snap;
+    int          m_measure_length;
+    int          m_beat_length;
 
     int          m_window_x, m_window_y;
  
@@ -73,12 +75,23 @@ class perfroll : public Gtk::DrawingArea
 
     int          m_roll_length_ticks;
 
+    int          m_drop_x, m_drop_y;
+
+    long         m_drop_tick;
+    long         m_drop_tick_trigger_offset;
+    int          m_drop_sequence;
+
+    bool         m_sequence_active[c_total_seqs];
+    
     Adjustment   *m_vadjust;
     Adjustment   *m_hadjust;
 
     bool m_adding;
     bool m_adding_pressed;
     bool m_adding_pressed_state;
+    bool m_moving;
+    bool m_growing;
+    bool m_grow_direction;
 
     void on_realize();
     bool on_expose_event(GdkEventExpose* a_ev);
@@ -90,24 +103,32 @@ class perfroll : public Gtk::DrawingArea
     bool on_focus_out_event(GdkEventFocus*);
 
     void on_size_request(GtkRequisition* );
-    void on_size_allocate(GtkAllocation* );
+    void on_size_allocate(Gtk::Allocation& );
+
+    bool on_key_press_event(GdkEventKey* a_p0);
 
     void convert_xy( int a_x, int a_y, long *a_ticks, int *a_seq);
+    void convert_x( int a_x, long *a_ticks);
     void snap_x( int *a_x );
 
-    void draw_sequence_on( Glib::RefPtr<Gdk::Drawable> a_draw, int a_sequence );
-    void draw_background_at( Glib::RefPtr<Gdk::Drawable> a_draw, int a_x, long a_y );
+   
 
+    void draw_sequence_on( Glib::RefPtr<Gdk::Drawable> a_draw, int a_sequence );
+    void draw_background_on( Glib::RefPtr<Gdk::Drawable> a_draw, int a_sequence );
+
+    void draw_drawable_row( Glib::RefPtr<Gdk::Drawable> a_dest, Glib::RefPtr<Gdk::Drawable> a_src,  long a_y );
+    
+    
     void change_horz( void );
     void change_vert( void );
 
 
 
  public:
-
-    void set_snap( int a_snap );
+    void set_guides( int a_snap, int a_measure, int a_beat );
 
     void update_sizes();
+    void init_before_show( );
     void fill_background_pixmap();
 
     void increment_size();
@@ -116,6 +137,8 @@ class perfroll : public Gtk::DrawingArea
 
     void set_adding( bool a_adding );
 
+    void redraw_dirty_sequences( void );
+
     perfroll( perform *a_perf, 
 	      Adjustment *a_hadjust,
 	      Adjustment *a_vadjust );
