@@ -91,7 +91,7 @@ seqevent::on_realize()
     m_gc = Gdk::GC::create( m_window );
     m_window->clear();
 
-    m_hadjust->signal_value_changed().connect( slot( *this, &seqevent::change_horz ));
+    m_hadjust->signal_value_changed().connect( mem_fun( *this, &seqevent::change_horz ));
 
     update_sizes();
 }
@@ -110,13 +110,13 @@ seqevent::change_horz( )
 
 
 void
-seqevent::on_size_allocate(GtkAllocation* a_r )
+seqevent::on_size_allocate(Gtk::Allocation& a_r )
 {
     Gtk::DrawingArea::on_size_allocate( a_r );
     
-    m_window_x = a_r->width;
-    m_window_y = a_r->height;
-    
+    m_window_x = a_r.get_width();
+    m_window_y = a_r.get_height();
+  
     update_sizes(); 
  
 }
