@@ -28,7 +28,6 @@ seqroll::seqroll(perform *a_perf,
                  seqdata *a_seqdata_wid, 
                  seqevent *a_seqevent_wid,
                  seqkeys *a_seqkeys_wid,
-                 mainwid *a_mainwid,
                  int a_pos, 
                  Adjustment *a_hadjust,
                  Adjustment *a_vadjust )
@@ -56,7 +55,6 @@ seqroll::seqroll(perform *a_perf,
     m_snap =  a_snap;
     m_seqdata_wid = a_seqdata_wid;
     m_seqevent_wid = a_seqevent_wid;
-    m_mainwid = a_mainwid;
     m_seqkeys_wid = a_seqkeys_wid;
     m_pos = a_pos;
 
@@ -151,8 +149,8 @@ seqroll::on_realize()
     m_gc = Gdk::GC::create( m_window );
     m_window->clear();
 
-    m_hadjust->signal_value_changed().connect( slot( *this, &seqroll::change_horz ));
-    m_vadjust->signal_value_changed().connect( slot( *this, &seqroll::change_vert ));
+    m_hadjust->signal_value_changed().connect( mem_fun( *this, &seqroll::change_horz ));
+    m_vadjust->signal_value_changed().connect( mem_fun( *this, &seqroll::change_vert ));
 
     update_sizes();
 }
@@ -201,8 +199,6 @@ seqroll::update_sizes()
     
     /* create pixmaps with window dimentions */
     if( is_realized() ) {
-
-    
         
         m_pixmap = Gdk::Pixmap::create( m_window,
                                         m_window_x,
@@ -244,7 +240,6 @@ seqroll::reset()
     update_sizes();
     update_pixmap();
     queue_draw();
-    m_mainwid->update_sequence_on_window( m_pos );
 }
 
 void
@@ -255,7 +250,6 @@ seqroll::redraw()
     
     update_pixmap();
     queue_draw();
-    m_mainwid->update_sequence_on_window( m_pos );
 }
 
 
@@ -989,8 +983,6 @@ seqroll::on_button_press_event(GdkEventButton* a_ev)
     if ( needs_update ){
         update_pixmap();
         queue_draw();
-        m_mainwid->update_sequence_on_window( m_pos );
-        
         m_seqevent_wid->redraw(); 
         m_seqdata_wid->redraw();
     }
@@ -1089,9 +1081,7 @@ seqroll::on_button_release_event(GdkEventButton* a_ev)
     if (  needs_update ){
         
         update_pixmap();
-        queue_draw();
-        m_mainwid->update_sequence_on_window( m_pos );
-        
+        queue_draw();        
         m_seqevent_wid->redraw(); 
         m_seqdata_wid->redraw();
     }
@@ -1205,45 +1195,43 @@ seqroll::on_key_press_event(GdkEventKey* a_p0)
 
 	if ( a_p0->keyval ==  GDK_Delete ){
 
-		m_seq->push_undo();
+            m_seq->push_undo();
 	    m_seq->remove_selected();
 	    ret = true;
 	}
-
+        
 	if ( a_p0->state & GDK_CONTROL_MASK ){
-
+            
 	    /* cut */
 	    if ( a_p0->keyval == GDK_x || a_p0->keyval == GDK_X ){
 		
-			m_seq->push_undo();
-			m_seq->copy_selected();
-			m_seq->remove_selected();
-
-			ret = true;
+                m_seq->push_undo();
+                m_seq->copy_selected();
+                m_seq->remove_selected();
+                
+                ret = true;
 	    }
 	    /* copy */
 	    if ( a_p0->keyval == GDK_c || a_p0->keyval == GDK_C ){
 		
-			m_seq->copy_selected();
-			m_seq->unselect();
-			ret = true;
+                m_seq->copy_selected();
+                m_seq->unselect();
+                ret = true;
 	    }
-
-       /* paste */
-		if ( a_p0->keyval == GDK_v || a_p0->keyval == GDK_V ){
-                                                                   
-			start_paste();
-			ret = true;
-		}
+            
+            /* paste */
+            if ( a_p0->keyval == GDK_v || a_p0->keyval == GDK_V ){
+                
+                start_paste();
+                ret = true;
+            }
 	}
     }
 
     if ( ret == true ){
         
         update_pixmap();
-        queue_draw();
-        m_mainwid->update_sequence_on_window( m_pos );
-        
+        queue_draw();        
         m_seqevent_wid->redraw(); 
         m_seqdata_wid->redraw();
         return true;
@@ -1266,12 +1254,12 @@ seqroll::set_data_type( unsigned char a_status, unsigned char a_control = 0 )
 
 
 void
-seqroll::on_size_allocate(GtkAllocation* a_r )
+seqroll::on_size_allocate(Gtk::Allocation& a_r )
 {
     Gtk::DrawingArea::on_size_allocate( a_r );
 
-    m_window_x = a_r->width;
-    m_window_y = a_r->height;
+    m_window_x = a_r.get_width();
+    m_window_y = a_r.get_height();
 
     update_sizes(); 
  
