@@ -80,8 +80,8 @@ seqtime::on_realize()
     // we need to do the default realize
     Gtk::DrawingArea::on_realize();
 
-    //Gtk::Main::idle.connect(slot(this,&seqtime::idleProgress));
-    Glib::signal_timeout().connect(slot(*this,&seqtime::idle_progress), 50);
+    //Gtk::Main::idle.connect(mem_fun(this,&seqtime::idleProgress));
+    Glib::signal_timeout().connect(mem_fun(*this,&seqtime::idle_progress), 50);
 
   
 
@@ -91,7 +91,7 @@ seqtime::on_realize()
     m_gc = Gdk::GC::create( m_window );
     m_window->clear();
 
-    m_hadjust->signal_value_changed().connect( slot( *this, &seqtime::change_horz ));
+    m_hadjust->signal_value_changed().connect( mem_fun( *this, &seqtime::change_horz ));
 
     update_sizes();
 }
@@ -110,12 +110,12 @@ seqtime::change_horz( )
 
 
 void
-seqtime::on_size_allocate(GtkAllocation* a_r )
+seqtime::on_size_allocate(Gtk::Allocation & a_r )
 {
     Gtk::DrawingArea::on_size_allocate( a_r );
     
-    m_window_x = a_r->width;
-    m_window_y = a_r->height;
+    m_window_x = a_r.get_width();
+    m_window_y = a_r.get_height();
     
     update_sizes(); 
  
